# frozen_string_literal: true
#
# Copyright:: Copyright (c) Chef Software Inc.
# License:: Apache License, Version 2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

Ohai.plugin(:Train) do
  provides "train"

  collect_data do
    if transport_connection
      train Mash.new
      train["family_hierarchy"] = transport_connection.platform.family_hierarchy
      train["family"] = transport_connection.platform.family
      train["platform"] = transport_connection.platform.platform
      train["backend"] = transport_connection.backend_type
      if transport_connection.respond_to?(:uri)
        train["scheme"] = URI.parse(transport_connection.uri).scheme
        train["uri"] = transport_connection.uri
      end
    end
  end
end
