# frozen_string_literal: true
#
# Copyright:: Copyright (c) Chef Software Inc.
# License:: Apache License, Version 2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

require "chef-utils/dsl/train_helpers" unless defined?(ChefUtils::DSL::TrainHelpers)
require_relative "chef_utils_wiring" unless defined?(Ohai::Mixin::ChefUtilsWiring)

module Ohai
  module Mixin
    module TrainHelpers
      include ChefUtils::DSL::TrainHelpers
      include ChefUtilsWiring

      # anything added to this file temporarily should be pushed back up
      # into ChefUtils::DSL::TrainHelpers

      # XXX: this needs better support directly in train
      def dir_glob(path)
        shell_out!("ls -d #{path}").stdout.split
      end
    end
  end
end
